# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Productivity module with general infrastructure support
"""
from EMAN2 import EMData, EMUtil, display
from getpass import getuser
import math
import os
import shutil
import signal
import subprocess
import sys
import time

from PyQt4.QtCore import QProcess, QProcessEnvironment, QIODevice, QThread
from tabulate import tabulate

import matplotlib.pyplot as plt
import numpy as np
from compiler.ast import nodes


class DiagnosticPlot(object):

    def __init__(self, fig_no=None):
        if fig_no is None:
            start_fig = 0 
        else:
            start_fig = fig_no
            
        self.fig_no = iter(list(range(start_fig, 10000)))
        self.plt = plt
        self.fig = self.plt.figure(fig_no)


    def get_empty_fig(self):
        
        return self.fig
    
    
    def create_next_figure(self):
        self.fig = self.plt.figure(self.fig_no.next())
        
        return self.fig
    
        
    def add_header_and_footer(self, feature_set=None, infile=None, outfile=None):
        """
        * Function to add header for matplotlib output plots
        
        #. Input: infile name, outfile name to be added to header
        #. Output: figure with header 
        #. Usage: fig = DiagnosticPlot(fig).add_header_and_footer(feature_set, \
        infile, outfile)
        """
        if feature_set is None: 
            feature_set = self.feature_set
        if infile is None: 
            infile = list(feature_set.parameters.values())[0]
        if outfile is None: 
            outfile = list(feature_set.parameters.values())[1]

        self.fig.suptitle('{prg} {infile} {outfile}'.format(prg=feature_set.progname,
        infile=os.path.basename(list(feature_set.parameters.values())[0]), outfile=os.path.basename(outfile)))

        self.fig.subplots_adjust(left=None, bottom=0.3, right=None, top=None, wspace=0.3, hspace=None)
        parameters = parameter_values = ''
        for name in feature_set.parameters:
            parameters += '{0:<80}\n'.format(name) 
            if str(feature_set.parameters[name]).startswith(os.sep):
                filename_split = feature_set.parameters[name].split(os.sep)
                value = (os.sep.join(filename_split[-3:]))
            else:
                value = feature_set.parameters[name]
            parameter_values += '= {0}\n'.format(value)

        parameters += '{ltime} (PID={pid})'.format(ltime=time.strftime('%a, %d %b %Y %H:%M:%S', time.localtime()),
                                             pid=os.getpid())
        parameter_values += '   Printed to {0}'.format(os.path.basename(outfile))
        
        if len(feature_set.parameters) < 12:
            self.fig.text(0.1, 0.03, parameters, fontsize=4)
            self.fig.text(0.43, 0.03, parameter_values, fontsize=4)
        else:
            self.fig.text(0.1, 0.03, parameters, fontsize=3)
            self.fig.text(0.43, 0.03, parameter_values, fontsize=3)

        return self.fig


    def set_fontsize_to_all_ticklabels_of_subplots(self, subplot_collection, font_size=4):
        for each_subplot in subplot_collection:
            for t in each_subplot.get_xticklabels():
                t.set_fontsize(font_size)
            
            for t in each_subplot.get_yticklabels():
                t.set_fontsize(font_size)
                
        return subplot_collection
    
    def __del__(self):
        plt.close()
        

class Support(object):
    def define_levels_beginner_intermediate_expert(self):
        levels = {'beginner':['beginner'], 
                  'intermediate':['beginner', 'intermediate'], 
                  'expert':['beginner', 'intermediate', 'expert']}
        
        return levels
    
    
    def search_path_like_which(self, exe):
        for each_dir in os.environ['PATH'].split(os.pathsep):
            binpath = os.path.join(each_dir, exe) 
            if os.path.exists(binpath):
                return os.path.abspath(binpath)
        return None
    
    
    def compute_byte_size_of_image_stack(self, xsize, ysize, image_count):
        """
        >>> a = np.polyfit([1, 10000, 500**2], [10124, 50120, 1010120], 2) 
        >>> np.polyval(a, 300 * 300)
        370119.99999999924
        >>> from spring.csinfrastr.csproductivity import Support
        >>> Support().compute_byte_size_of_image_stack(300, 300, 1)
        370120
        >>> Support().compute_byte_size_of_image_stack(300, 300, 10)
        3610120
        """
        header_size =  10120
        byte_size =  header_size + image_count * xsize * ysize * 4
        
        return byte_size
    
    
class ExtLauncher(QThread):
    """
    Class that holds functions for external system viewers
    """
    def __init__(self, parent=None):
        QThread.__init__(self, parent)
        self.qlaunchproc = set([])
        

    def construct_command_line(self, ofile):
        from platform import system
        ofileext = os.path.splitext(str(ofile))[-1].strip(os.extsep)
        args = None
        try:
            if ofileext in ['mrc', 'spi', 'hdf', 'img', 'hed']:
                prg = Support().search_path_like_which('springenv')
                args =  Support().search_path_like_which('e2display.py')
            elif system() == 'Darwin':
                prg = 'open'
            elif os.name == 'nt':
                prg = 'start'
            elif system() == 'Linux':
                prg = 'gnome-open'
        except:
            print('Platform not recognized or supported')
        if args is None:
            dirprogname = '{0} {1}'.format(prg, ofile)
        else:
            dirprogname = '{0} {1} {2}'.format(prg, args, ofile)
            
        print(dirprogname)
        
        return prg, args


    def display_em(self, em_file):
        display(em_file)
        
        
    def qlaunch(self, prg, args):
        QProcessEnvironment.systemEnvironment()
        self.proc = QProcess(self)
        self.proc.setProcessChannelMode(QProcess.MergedChannels)
        self.proc.start(prg, args, QIODevice.ReadWrite)
        self.qlaunchproc.add(self.proc) 
        self.proc.finished.connect(lambda: self.qlaunchproc.remove(self.proc))
        
    def finished(self):
        self.proc.terminate()


    def qlaunch_open_file(self, ofile):
        prg, add_args = self.construct_command_line(ofile)
        if add_args is None:
            args = [ofile]
        else: 
            args = [add_args, ofile]

        self.qlaunch(prg, args)


    def __del__(self):
        self.exiting = True
        self.wait()
        
        
class IOFormatError(IOError):
    pass

class Errors(object):
    def check_valerr(self, value):
        try:
            value = float(value)
            if value < self.properties[self.name].minimum or value > self.properties[self.name].maximum:
                raise ValueError()
        except ValueError:
            error_message = '{0}: value \'{1}\' '.format(self.feature_set.progname, self.name) + \
                             '{0} is outside the pre-defined input range (min={1}, max={2}).'.format(value, 
                              self.properties[self.name].minimum, self.properties[self.name].maximum)
            sys.stderr.write(error_message)
            raise ValueError(error_message)
        

    def check_filerr(self, file_name):
        try:
            if self.name in self.relatives and self.parameters[self.relatives[self.name]] is False:
                pass
            elif self.parameters[self.name].find('nosetest_') >= 0:
                print('File was generated by nosetest suite. Test is fine. Exit')
                sys.exit()
            elif file_name in ['.']:
                pass
            elif os.path.exists(file_name):
                print('{0} found'.format(file_name))
            elif not os.path.exists(file_name):
                raise IOError
        except IOError:
            error_message = 'File or directory \'{0}\' was not found.'.format(file_name)
            sys.stderr.write(error_message)
            raise IOError(error_message)
        self.check_fileformat(file_name)


    def check_fileformat(self, file_name):
        try:
            if self.properties[self.name].ext[0] == '*':
                pass
            elif self.name in self.relatives and self.parameters[self.relatives[self.name]] is False:
                pass
            elif file_name.split(os.extsep)[-1] not in self.properties[self.name].ext:
                raise IOFormatError()
        except IOFormatError:
            error_message = 'File format \'{0}\' for \'{1}\' is not supported.'.format(file_name.split(os.extsep)[-1], 
                                                                              self.name)
            sys.stderr.write(error_message)
            raise IOError(error_message)

        
"""
Disk Space
Simple class for working out the free disk space on a system

by Stuart Colville http://muffinresearch.co.uk
License: http://www.opensource.org/licenses/mit-license.php

"""


class DiskSpace():

    """Free Disk Space"""

    def __init__(self, path='/'):
        """Init class and retrieves disk space info"""
        self.disk = os.statvfs(path)

    def has_free_space(self, limit):
        """Bool returns true if remaining space is above limit %"""
        if float(self.percent_free()) < float(limit):
            return False
        else:
            return True

    def percent_free(self):
        """Gets the amount of space left as a percentage"""
        return (math.ceil(float(100) / float(self.bytes_capacity())
                                                * self.bytes_free()))

    def bytes_capacity(self):
        """Returns the total capacity in bytes"""
        return self.disk.f_frsize * self.disk.f_blocks

    def bytes_free(self):
        """Returns the free space in bytes"""
        return self.disk.f_frsize * self.disk.f_bavail

    def bytes_used(self):
        """Returns the used space in bytes"""
        return self.disk.f_frsize * (self.disk.f_blocks -
                                                   self.disk.f_bavail)

    @staticmethod
    def humanize_bytes(bytes, kilo=1024):

        """Humanizes bytes

        See http://en.wikipedia.org/wiki/Kilobyte for info on the
        different ways to interpret whether a kilobyte is 1,024 bytes
        or 1,000 bytes

        >>> from spring.csinfrastr.csproductivity import DiskSpace
        >>> DiskSpace().humanize_bytes(1024 ** 2)
        '1.00MB'
        >>> DiskSpace().humanize_bytes(1024 ** 3)
        '1.00GB'
        >>> DiskSpace().humanize_bytes(1024 ** 4)
        '1.00TB'
        """
        if kilo != 1024:
            kilo = 1000
            
        label = ['KB', 'MB' , 'GB', 'TB']

        if bytes < kilo:
            human_bytes = '%d' % bytes
        elif bytes >= kilo and bytes < math.pow(kilo, 2):
            human_bytes = '%.2F%s' % (float(bytes/kilo), label[0])
        elif bytes >= math.pow(kilo, 2) and bytes < math.pow(kilo, 3):
            human_bytes = '%.2F%s' % (float(bytes/math.pow(kilo, 2)), label[1])
        elif bytes >= math.pow(kilo, 3) and bytes < math.pow(kilo, 4):
            human_bytes = '%.2F%s' % (float(bytes/math.pow(kilo, 3)), label[2])
        elif bytes >= math.pow(kilo, 4) and bytes < math.pow(kilo, 5):
            human_bytes = '%.2F%s' % (float(bytes/math.pow(kilo, 4)), label[3])
        
        return human_bytes
        
        
class Temporary(object):
    def remove_dir_and_everything_inside(self, tempdir):
        if os.path.isdir(tempdir):
            files_to_be_deleted = os.listdir(tempdir)
            [os.remove(os.path.join(tempdir, each_file)) for each_file in files_to_be_deleted]
            shutil.rmtree(tempdir)


    def generate_tmpdir_name(self, temppath):
        tmpdir = temppath + os.sep + getuser() + '-' + time.strftime('%d_%b_%Y_%H_%M_%S') + '-' + '%d' % os.getpid() + \
        os.sep
        
        return tmpdir


    def check_available_space_in_curdir_and_add_to_error_if_not_enough_space_available(self):
        ds = DiskSpace(os.curdir)
        if ds.bytes_free() < 100000:
            err_msg = '\nIt looks like you ran out of disk space because currently only ' + \
            '{0} are available in the '.format(ds.humanize_bytes(ds.bytes_free())) + \
            'current directory {0}.'.format(os.path.abspath(os.curdir))
        else:
            err_msg = ''
        
        return err_msg


    def check_available_space_in_temppath_and_raise_error_if_not_enough_space_available(self, temppath, size):
        err_msg = None
        if size is not None:
            ds = DiskSpace(temppath)
            if 1.1 * size > ds.bytes_free():
                this_node_name = os.uname()[1]
                err_msg = 'The specified \'Temporary directory\' \'{0}\' does not have sufficient space '.format(temppath) + \
                'to carry out the task. Only {0} are available '.format(ds.humanize_bytes(ds.bytes_free())) + \
                'on {0}. The program is projected to require a minimum of '.format(this_node_name) + \
                '{0} in the temporary directory. '.format(ds.humanize_bytes(1.1 * size)) + \
                '\nSolutions: (1) Tile the job further ' + \
                'to run on more nodes which will distribute the temporary data onto more hard drives. ' + \
                '\n(2) The required disk space directly correlates with the size and number of requested projections. ' +\
                'Reduce the \'Alignment size\' or \'Number of projections azimuthal/out-of-plane angle\' parameters ' +\
                'if necessary. ' 
                
                if ds.bytes_capacity() > size:
                    err_msg += '\n(3) Clean-up the \'Temporary directory\' \'{0}\' '.format(temppath) + \
                    'on {0} because the drive has '.format(this_node_name) + \
                    '{0} of total capacity. '.format(ds.humanize_bytes(ds.bytes_capacity()))
                    err_msg += '\n(4) Remove other jobs that may concurrently fill up the temporary disk space. '
                    
                err_msg += '\n* If you do not know what all of ' + \
                'above means or if you do not have sufficient temporary disk space available choose ' + \
                'an alternative location for this task, e.g. the current directory \'.\' ' + \
                'Be aware, in case you are using distributed computing on multiple nodes, ' + \
                'this action will lead to unnecessary network traffic and slow down your ' + \
                'entire network significantly.'
            
        return err_msg


    def mkdir_temporary_directory(self, temppath, size=None):
        
        err_msg = self.check_available_space_in_temppath_and_raise_error_if_not_enough_space_available(temppath, size)
        tmpdir = self.generate_tmpdir_name(temppath)
        
        os.mkdir(tmpdir)
        
        return tmpdir, err_msg


    def mktmpdir(self, temppath, byte_size=None):
        tmpdir, err_msg = self.mkdir_temporary_directory(temppath, byte_size)
        
        _excepthook = sys.excepthook 

        def excepthook(t,v,tb): 
            _excepthook(t,v,tb) 
                    
            self.remove_dir_and_everything_inside(tmpdir)
            err_msg = self.check_available_space_in_curdir_and_add_to_error_if_not_enough_space_available()
            err_msg += '\nThe following temporary directory was cleared and removed:\n'
            err_msg += tmpdir
                    
            sys.stderr.write(err_msg)
            sys.exit(1)
        
        sys.excepthook = excepthook 
        
        return tmpdir
    
    
    def get_deletion_info_named_tuple(self):
        from collections import namedtuple
        del_i = namedtuple('del_info', 'rank_id node process_id directory space')
        
        return del_i
    

    def generate_tempdir_from_user(self, tempdir, this_user):
        return '{tmpdir}{sep}{user}*'.format(tmpdir=tempdir, user=this_user, sep=os.sep)
    

    def generate_deletion_info_from_nodefile(self, nodefile_string, tempdir, space=3, this_user=None):
        """
        >>> from spring.csinfrastr.csproductivity import Temporary
        >>> t = Temporary()
        >>> t.generate_deletion_info_from_nodefile(['cl1', 'cl2'], '/tmp', 3, 'me') #doctest: +NORMALIZE_WHITESPACE
         [del_info(rank_id=0, node='cl1', process_id=None, directory='/tmp/me*', space=3), 
         del_info(rank_id=1, node='cl2', process_id=None, directory='/tmp/me*', space=3)]
        """
        if this_user is None:
            this_user = getuser()
        
        del_i = self.get_deletion_info_named_tuple()
        
        nodelist = [del_i(each_node_index, each_node.strip(), None, 
        self.generate_tempdir_from_user(tempdir, this_user), space) for each_node_index, each_node in \
        enumerate(nodefile_string)] 
        
        return nodelist


    def read_through_every_logline_and_extract_deletion_info(self, logfile_lines):
        """
        >>> from spring.csinfrastr.csproductivity import Temporary
        >>> t = Temporary()
        >>> t.read_through_every_logline_and_extract_deletion_info(['Number of CPUs = 2', \
'rank_id node_name       process_id      directory    space',\
'0       clnode215.embl.de       8291    /tmp/sachse-11_Jul_2012_14_31_02-8291/    3.95',\
'1       clnode215.embl.de       8292    /tmp/sachse-11_Jul_2012_14_31_05-8292/    3.95']) #doctest: +NORMALIZE_WHITESPACE
        [del_info(rank_id='0', node='clnode215.embl.de', process_id='8291', 
        directory='/tmp/sachse-11_Jul_2012_14_31_02-8291/', space='3.95'), 
        del_info(rank_id='1', node='clnode215.embl.de', 
        process_id='8292', directory='/tmp/sachse-11_Jul_2012_14_31_05-8292/', space='3.95')]
        """
        del_i = self.get_deletion_info_named_tuple()
        
        browse = False
        cpu_count = None
        nodelist = []
        for each_line in logfile_lines:
            if cpu_count is not None:
                if browse:
                    nodelist.append(del_i._make(each_line.split()))
                    if len(nodelist) >= cpu_count:
                        browse = False
            if each_line.find('Number of CPUs') >= 0:
                cpu_count = int(each_line.split('=')[-1])
            if each_line.find('node_name') > 0:
                browse = True
        
        return nodelist
    

    def generate_deletion_info_from_start_end_node(self, start, end, nodes_prefix, tempdir, space, this_user=None):
        """
        >>> from spring.csinfrastr.csproductivity import Temporary
        >>> t = Temporary()
        >>> t.generate_deletion_info_from_start_end_node(1,4,'cl','/tmp', 3.98, 'me') #doctest: +NORMALIZE_WHITESPACE
        [del_info(rank_id=0, node='cl1', process_id=None, directory='/tmp/me*', space=3.98), 
        del_info(rank_id=1, node='cl2', process_id=None, directory='/tmp/me*', space=3.98), 
        del_info(rank_id=2, node='cl3', process_id=None, directory='/tmp/me*', space=3.98), 
        del_info(rank_id=3, node='cl4', process_id=None, directory='/tmp/me*', space=3.98)]
        """
        if this_user is None:
            this_user = getuser()
        del_i = self.get_deletion_info_named_tuple()
        nodelist = [del_i(each_node_index, '{0}{1}'.format(nodes_prefix, each_node_id), None, 
        self.generate_tempdir_from_user(tempdir, this_user), space) for each_node_index, each_node_id in \
        enumerate(list(range(start, end + 1)))] 
        
        return nodelist
    
    
class OpenMpiChecks(object):
    def check_dir(self, directory):
        """
        * Function to check existence of directory and rename if existent to \
        make room for next functions

        #. Input: arbitrary number of directories
        #. Output: 
        #. Usage: checkdir('this_directory')
        """
        if os.path.isdir(directory):
            from spring.csinfrastr.cslogger import Logger
            self.log = Logger()
            newdirectory = time.strftime(directory + time.strftime('_%d_%b_%Y_%H_%M_%S', 
                                            time.localtime( os.path.getctime(directory) )))
            os.rename(directory, newdirectory)
            self.log.ilog('{0} was renamed to {1} to make room for following program'.format(directory, newdirectory))
            directory = newdirectory 
        
        return directory


    def check_expected_output_file(self, program_to_be_launched, file_expected):
        if not os.path.isfile(file_expected):
            error_message = '{prg} did not finish successfully. The output '.format(prg=program_to_be_launched) + \
            'file {output} was not found, please check logfile of {prg} for details.'.format(output=file_expected, 
            prg=program_to_be_launched)
            raise IOError(error_message)
        

class OpenMpiLaunch(OpenMpiChecks):
    """
    Class that holds functions for OpenMpi usage
    """
    def get_nodes_from_pe_hostfile(self, hostfile):
        """
        >>> hostfile_str = 'node1.embl.de 3 R815.q@node1.embl.de UNDEFINED\\nnode2.embl.de 2 R815.q@node.embl.de UNDEFINED'
        >>> t = open('testhostfile.dat', 'w')
        >>> t.write(hostfile_str)
        >>> t.close()
        >>> from spring.csinfrastr.csproductivity import OpenMpi
        >>> OpenMpi().get_nodes_from_pe_hostfile('testhostfile.dat')
        ['node1.embl.de', 'node1.embl.de', 'node1.embl.de', 'node2.embl.de', 'node2.embl.de']
        >>> os.remove('testhostfile.dat')
        """
        file_lines = [n.split() for n in open(hostfile)]
        nodes = [each_line[0] for each_line in file_lines for each_cpu in list(range(int(each_line[1])))]

        return nodes


    def get_nodes_from_LSB_MCPU_HOSTS_variable(self, variable):
        """
        >>> variable = 'compute082 3 compute089 4 compute-n065 10'
        >>> from spring.csinfrastr.csproductivity import OpenMpi
        >>> OpenMpi().get_nodes_from_LSB_MCPU_HOSTS_variable(variable) #doctest: +NORMALIZE_WHITESPACE
        ['compute082', 'compute082', 'compute082', 'compute089', 'compute089', 
        'compute089', 'compute089', 'compute-n065', 'compute-n065', 
        'compute-n065', 'compute-n065', 'compute-n065', 'compute-n065', 
        'compute-n065', 'compute-n065', 'compute-n065', 'compute-n065'] 
        """
        vars = variable.split()
        nodes = [vars[2 * each_var] for each_var in range(len(vars)/2) for each_node in range(int(vars[2 * each_var + 1]))]
        
        return nodes


    def get_nodes_from_hostname_SLURM(self, stdout):
        """
        >>> output = '02: bc02-01.cluster.embl.de\\n01: bc02-01.cluster.embl.de\\n08: bc02-02.cluster.embl.de'
        >>> from spring.csinfrastr.csproductivity import OpenMpi
        >>> OpenMpi().get_nodes_from_hostname_SLURM(output)
        ['bc02-01.cluster.embl.de', 'bc02-01.cluster.embl.de', 'bc02-02.cluster.embl.de']
        """
        lines = [each_line for each_line in stdout.split('\n') if each_line != '']
        lines.sort()
        nodes = [each_line.split()[-1] for each_line in lines]
        
        return nodes
    

    def get_nodes_from_SLURM_by_running_srun_hostname(self):
        srun_process = subprocess.Popen('srun -l /bin/hostname', stdout=subprocess.PIPE, shell=True)
        stdout, log_stderr = srun_process.communicate()
        srun_process.wait()
 
        nodes = self.get_nodes_from_hostname_SLURM(stdout)
         
        return nodes


    def add_mpi_cmds(self, command_line_string, CPU_count):
        abspath_main = Support().search_path_like_which(command_line_string.split()[0])
        command_line_string = ' '.join([abspath_main] + command_line_string.split()[1:])
        abspath_mpirun = Support().search_path_like_which('mpirun')
        if os.getenv('PBS_NODEFILE') or os.getenv('PE_HOSTFILE') or \
        os.getenv('LSB_MCPU_HOSTS'): 
            if os.getenv('PBS_NODEFILE'):
                nodes = [n.strip() for n in open(os.environ['PBS_NODEFILE'])]
            elif os.getenv('PE_HOSTFILE'):
                nodes = self.get_nodes_from_pe_hostfile(os.environ['PE_HOSTFILE'])
            elif os.getenv('LSB_MCPU_HOSTS'):
                nodes = self.get_nodes_from_LSB_MCPU_HOSTS_variable(os.environ['LSB_MCPU_HOSTS'])
            nodefile_name = 'nodefile.dat'
            nodefile = open(nodefile_name, 'w')
            for each_node in nodes:
                nodefile.write('{0}\n'.format(each_node))
            nodefile.close()
            
            preargs = '{mpirun} -v --hostfile {nodefile} -np {nocpu} '.format(mpirun=abspath_mpirun, 
            nodefile=nodefile_name, nocpu=len(nodes))
        elif os.getenv('SLURM_JOB_NODELIST'):
#             nodes = self.get_nodes_from_SLURM_by_running_srun_hostname()
            preargs = '{0} '.format(abspath_mpirun)
        else:
            from multiprocessing import cpu_count
            if CPU_count > cpu_count():
                CPU_count = cpu_count()
            preargs = '{mpirun} -np {nocpu} '.format(mpirun=abspath_mpirun, nocpu=CPU_count)
        command_line_string = preargs + command_line_string
        
        return command_line_string


    def launch_command(self, command_line_string):
        from spring.csinfrastr.cslogger import Logger
        self.log = Logger()
        self.log.tlog(command_line_string)
        myenv = os.environ.copy()
        myenv['PYTHONPATH'] = ':'.join(sys.path) + ':' + os.environ['PYTHONPATH']
        subprocess.call(command_line_string.split(), env=myenv)
    
    
    def check_if_mpi_works_and_launch_command(self, command_line_string, cpu_count):
        try:
            import mpi4py
        except ImportError:
            warnstr = 'Mpi4py not installed, program proceeds with single CPU.'
            print('\nWARNING: ' + warnstr)
        else:
            command_line_string = self.add_mpi_cmds(command_line_string, cpu_count)
        
        self.launch_command(command_line_string)


    def launch_mpi_version_of_program(self, cpu_count, feature_set, parfile):
        progname_mpi = feature_set.progname + '_mpi'
        abspath_logfile = feature_set.logfile 
        
        command_line_string = '{prg} --f {parfile} --d {directory} --l {logfile}'.format(prg = progname_mpi, 
        parfile=parfile, directory=os.path.curdir, logfile=abspath_logfile)

        self.check_if_mpi_works_and_launch_command(command_line_string, cpu_count)
        os.remove(parfile)


    def setup_and_start_mpi_version_if_demanded(self, mpi_option, feature_set, cpu_count, exiting=True):
        from spring.csinfrastr.csfeatures import Features
        if mpi_option is True:
            external_mpi_run = OpenMpi()
            parfile = Features().write_parfile(feature_set.parameters)
            external_mpi_run.launch_mpi_version_of_program(cpu_count, feature_set, parfile)
            
            cleanup_script = 'cleanup.py'
            if os.path.isfile(cleanup_script):
                abs_python = Support().search_path_like_which('python')
                command_line_string = '{0} {1} {2}'.format('springenv', abs_python, cleanup_script)
                external_mpi_run.check_if_mpi_works_and_launch_command(command_line_string, cpu_count)
                os.remove(cleanup_script)
                
            if exiting:
                sys.exit()


class OpenMpiPythonObjects(OpenMpiLaunch):
    def split_sequence_evenly(self, seq, size):
        """
        >>> from spring.csinfrastr.csproductivity import OpenMpi
        >>> OpenMpi().split_sequence_evenly(list(range(9)), 4)
        [[0, 1], [2, 3, 4], [5, 6], [7, 8]]
        >>> OpenMpi().split_sequence_evenly(list(range(18)), 4)
        [[0, 1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12, 13], [14, 15, 16, 17]]
        """
        newseq = []
        splitsize = 1.0/size*len(seq)
        for i in range(size):
            newseq.append(seq[int(round(i*splitsize)):int(round((i+1)*splitsize))])
        return newseq
    
    
    def merge_sequence_of_sequences(self, seq):
        """
        >>> from spring.csinfrastr.csproductivity import OpenMpi
        >>> OpenMpi().merge_sequence_of_sequences([list(range(9)), list(range(3))])
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 0, 1, 2]
        >>> OpenMpi().merge_sequence_of_sequences([list(range(9)), [], list(range(3))])
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 0, 1, 2]
        """
        newseq = [number for sequence in seq for number in sequence]
        
        return newseq
    
    
    def gather_named_tuples_of_naparrays_from_cpus_to_common_named_tuple(self, comm, named_tuple):
        new_tuple = []
        for each_tuple in list(named_tuple):
            gathered_tuples = comm.gather(each_tuple, root=0) 
            if comm.rank == 0:
                new_tuple.append(np.array(self.merge_sequence_of_sequences(gathered_tuples)))
        if comm.rank == 0:
            gathered_named_tuple = named_tuple._make(new_tuple)
        else:
            gathered_named_tuple = None
        
        return gathered_named_tuple
        
        
    def convert_list_of_namedtuples_to_list_of_lists(self, list_of_namedtuples):
        """
        >>> from collections import namedtuple
        >>> helix_items = 'micrograph segment_list directory coordinates'
        >>> helix_info = namedtuple('helixinfo', helix_items)
        >>> to_convert = [helix_info(1*i, 2*i, 3*i, 4*i) for i in range(5)]
        >>> from spring.segment2d.segment_mpi import SegmentMpi
        >>> OpenMpi().convert_list_of_namedtuples_to_list_of_lists(to_convert)
        [[0, 0, 0, 0], [1, 2, 3, 4], [2, 4, 6, 8], [3, 6, 9, 12], [4, 8, 12, 16]]
        >>> OpenMpi().convert_list_of_namedtuples_to_list_of_lists([helix_info(1*i, 2*i, 3*i, 4*i)])
        [[4, 8, 12, 16]]
        """
        list_of_lists = [ list(each_namedtuple) for each_namedtuple in list_of_namedtuples ]
        
        return list_of_lists


    def convert_list_of_lists_to_list_of_provided_namedtuple(self, list_of_lists, named_tuple):
        """
        >>> from collections import namedtuple
        >>> helix_items = 'micrograph segment_list directory coordinates'
        >>> helix_info = namedtuple('helixinfo', helix_items)
        >>> l = [[0, 0, 0, 0], [1, 2, 3, 4], [2, 4, 6, 8], [3, 6, 9, 12], [4, 8, 12, 16]]
        >>> o = OpenMpi()
        >>> o.convert_list_of_lists_to_list_of_provided_namedtuple(l, helix_info) #doctest: +NORMALIZE_WHITESPACE
        [helixinfo(micrograph=0, segment_list=0, directory=0, coordinates=0), 
        helixinfo(micrograph=1, segment_list=2, directory=3, coordinates=4), 
        helixinfo(micrograph=2, segment_list=4, directory=6, coordinates=8), 
        helixinfo(micrograph=3, segment_list=6, directory=9, coordinates=12), 
        helixinfo(micrograph=4, segment_list=8, directory=12, coordinates=16)]
        """
        list_of_namedtuples = [ named_tuple._make(each_list) for each_list in list_of_lists ]
        
        return list_of_namedtuples
    

class OpenMpiEMData(OpenMpiPythonObjects):
    def gather_stacks_from_cpus_to_common_stack(self, comm, local_stack, common_stack):
        """
        * Function to merge stacks from local disks to a single common stack
        """
        distributed_stacks = comm.gather(local_stack, root=0)
        distributed_stacks = comm.bcast(distributed_stacks, root=0)
        
        if comm.rank == 0:
            if os.path.isfile(common_stack):
                os.remove(common_stack)
            
        temp = EMData()
        for each_cpu, stack_of_each_cpu in enumerate(distributed_stacks):
            comm.barrier()
            if each_cpu == comm.rank and stack_of_each_cpu is not None:
                if os.path.isfile(stack_of_each_cpu):
                    local_stack_image_count = EMUtil.get_image_count(stack_of_each_cpu)
                    for each_local_image_index in range(local_stack_image_count):
                        temp.read_image(stack_of_each_cpu, each_local_image_index)
                        temp.append_image(common_stack)
                    os.remove(stack_of_each_cpu)
        
        return common_stack
    
    
    def transfer_series_of_images_from_cpus_to_common_disk(self, comm, local_images):
        """
        * Function to relocate series of local images/volumes to a common space in working directory
        """
        temp = EMData()
        images_in_common_dir = []
        for each_cpu in list(range(comm.size)):
            comm.barrier()
            for each_local_image in local_images:
                if each_cpu == comm.rank and os.path.isfile(each_local_image):
                    local_image_count = EMUtil.get_image_count(each_local_image)
                    image_in_common_directory = os.path.abspath(os.path.basename(each_local_image))
                    for each_local_image_index in range(local_image_count):
                        temp.read_image(each_local_image, each_local_image_index)
                        temp.write_image(image_in_common_directory, each_local_image_index)
                        images_in_common_dir.append(image_in_common_directory)
                        os.remove(each_local_image)
        
        comm.barrier()
        
        return images_in_common_dir
                
        
    def write_out_emdata_from_distributed_nodes_to_common_disk(self, comm, fftvol, file_name):
        """
        * Function to write out local images/volumes to a common space with node id in file suffix
        """
        accessible_local_emdata = os.path.splitext(file_name)
        
        emdata_file_name = '{prefix}{rank:03}{ext}'.format(prefix=accessible_local_emdata[0], rank=comm.rank,
        ext=accessible_local_emdata[-1])
        
        for each_cpu in list(range(comm.size)):
            comm.barrier()
            if each_cpu == comm.rank:
                fftvol.write_image(emdata_file_name)
        emdata_files = comm.gather(emdata_file_name, root=0)
        
        return emdata_files
    

    def reduce_emdata_on_main_node(self, emdata, distributed_emdata_files, read_first=False):
        temp = EMData()
        for each_index, each_file in enumerate(distributed_emdata_files):
            if not read_first and each_index == 0:
                pass
            else:
                if each_file is not None:
                    temp.read_image(each_file)
                    emdata += temp
            if each_file is not None:
                os.remove(each_file)
        
        emdata /= float(len(distributed_emdata_files))
        
        return emdata
    

class OpenMpi(OpenMpiEMData):

    def start_main_mpi(self, parset):
        from mpi4py import MPI
        comm = MPI.COMM_WORLD
        rank = comm.Get_rank()
        
        if rank == 0:
            # Option handling
            from spring.csinfrastr.csreadinput import OptHandler
            mergeparset = OptHandler(parset)
            parameters = mergeparset.parameters
            current_directory = os.path.abspath(os.path.curdir)
            logfile = mergeparset.logfile
            package = mergeparset.package
            progname = mergeparset.progname
            version = mergeparset.version
        else:
            parameters = None
            current_directory = None
            logfile = None
            package = None
            progname = None
            version = None
            
        parameters = comm.bcast(parameters, root=0)
        logfile = comm.bcast(logfile, root=0)
        package = comm.bcast(package, root=0)
        progname = comm.bcast(progname, root=0)
        version = comm.bcast(version, root=0)
        current_directory = comm.bcast(current_directory, root=0)
        if rank != 0:
            os.chdir(current_directory)
            
        from spring.csinfrastr.csfeatures import Features
        reduced_parset = Features().reduce_features_to_parameters_for_mpi(parameters)
        reduced_parset.logfile = logfile
        reduced_parset.package = package
        reduced_parset.progname = progname
        reduced_parset.version = version
        
        return reduced_parset
    

    def write_out_tempdir_record_and_cleanup_script(self, tempdir_str):
        
        cleanup_script = 'cleanup.py'
        cscript = open(cleanup_script, 'w')
        cscript.write('\'\'\'{0}\'\'\''.format(tempdir_str))
        cleanup_code = """
import sys
from mpi4py import MPI 
from spring.csinfrastr.csproductivity import Temporary
from tabulate import tabulate

comm = MPI.COMM_WORLD
rank = comm.rank

for each_lid, each_line in enumerate(__doc__.split('\\n')):
    rank_rec, host, pid, tempdir, space = each_line.split()
    if rank == int(rank_rec):
        Temporary().remove_dir_and_everything_inside(tempdir)
        temp_msg = [rank, host, pid, tempdir]
        
comm.barrier()
temp_msg = comm.gather(temp_msg, root = 0)

if rank == 0:
    header = ['rank', 'host', 'process_id', 'temporary directory']
    tab_msg = tabulate(temp_msg, header)
    msg = '\\nThe following temporary directories were cleared and removed:\\n{0}'.format(tab_msg)
    sys.stderr.write(msg)
    """
        cscript.write(cleanup_code)
        cscript.close()
        
        return cleanup_script
    

    def split_message_string_into_chunks_of_xx(self, msg, chunk_size=200):
        mmsg = [msg[i:i + chunk_size] for i in list(range(0, len(msg), chunk_size))]

        return mmsg


    def setup_mpi_abort_upon_exception_on_any_node(self, comm, tempdir_str, MPI):
        comm.Set_errhandler(MPI.ERRORS_ARE_FATAL)
        _excepthook = sys.excepthook 

        def excepthook(t,v,tb): 
            _excepthook(t,v,tb) 
            if not MPI.Is_finalized() and MPI.Is_initialized(): 
                err_msg = Temporary().check_available_space_in_curdir_and_add_to_error_if_not_enough_space_available()

                err_msg += '\nRank {0} on machine {1} with process ID {2} '.format(comm.rank, os.uname()[1], os.getpid())+\
                'raised an error message. Find the corresponding error message above. All other MPI processes are ' + \
                'aborted. Ignore the error messages thereafter as they can arise as a result of the aborting '+ \
                'procedure.\n'
                if tempdir_str is not None:
                    self.write_out_tempdir_record_and_cleanup_script(tempdir_str)
                    
                mmsg = self.split_message_string_into_chunks_of_xx(err_msg)
                sys.stderr.writelines(mmsg)
                MPI.COMM_WORLD.Abort(1) 
        sys.excepthook = excepthook 
        
        return comm
    

    def setup_interrupt(self, tempdir_str, size, MPI):
        abs_mpirun = Support().search_path_like_which('mpirun')
        abs_python = Support().search_path_like_which('python')
        nodefile_name = os.path.join(os.path.split(os.path.abspath(os.curdir))[-1], 'nodefile.dat')
        cleanup_script = os.path.join(os.path.split(os.path.abspath(os.curdir))[-1], 'cleanup.py')

        def interrupt_handler(signal, frame):
            if not MPI.Is_finalized() and MPI.Is_initialized(): 
                if tempdir_str is not None:
                    self.write_out_tempdir_record_and_cleanup_script(tempdir_str)
                    msg = '\n{0} was terminated. All files are kept. '.format(os.path.basename(sys.argv[0])) + \
                    'Please remember to clean up the temporary directories of this run on the respective nodes. ' + \
                    'For this purpose a cleanup script has been prepared - simply run:\n'

                    msg += '\'{0} -np {1} '.format(abs_mpirun, size)
                    if os.getenv('PBS_NODEFILE') or os.getenv('PE_HOSTFILE') or \
                    os.getenv('LSB_MCPU_HOSTS') or os.getenv('SLURM_JOB_NODELIST'):
                        msg +='--hostfile {0} '.format(nodefile_name)

                    msg += 'springenv {0} {1}\' in the current directory.\n'.format(abs_python, cleanup_script)

                    msg += '\nThe following temporary directories are affected:\n{0}\n'.format(tempdir_str)
                    mmsg = self.split_message_string_into_chunks_of_xx(msg)
                    sys.stderr.writelines(mmsg)
                MPI.COMM_WORLD.Abort(1) 
        
        signal.signal(signal.SIGINT, interrupt_handler)
        

    def get_node_names_and_broadcast_to_each_node(self, comm):
        this_node = os.uname()[1]
        nodes = comm.gather(this_node, root=0)
        nodes = comm.bcast(nodes, root=0)

        return nodes, this_node


    def get_job_current_count_on_this_node(self, comm):
        nodes, this_node = self.get_node_names_and_broadcast_to_each_node(comm)
        job_count_on_this_node = len([True for each_node in nodes if each_node == this_node])
        
        unique_nodes = list(set(nodes))
        
        return job_count_on_this_node, this_node, unique_nodes
        

    def get_first_local_tempdir(self, comm, tempdir):
        nodes, this_node = self.get_node_names_and_broadcast_to_each_node(comm)

        nodes = np.array(nodes)
        tempdirs = comm.gather(tempdir, root=0)
        tempdirs = np.array(comm.bcast(tempdirs, root=0))
        first_local_tempdir = tempdirs[nodes == this_node][0]

        return first_local_tempdir


    def make_tempdir_and_log_including_node_name(self, log, temppath, comm, rank, size, byte_size=None):
        if temppath is not None:
            from spring.csinfrastr.cslogger import Logger
            self.log = Logger()

            this_node = os.uname()[1]
            if byte_size is None:
                byte_size = 1024 ** 3
                
            for each_rank in list(range(size)):
                if each_rank == rank:
                    tempdir, err_msg = Temporary().mkdir_temporary_directory(temppath, byte_size)
                comm.barrier()
                
            first_local_tempdir = self.get_first_local_tempdir(comm, tempdir)
            if not first_local_tempdir.startswith(tempdir):
                byte_size = Support().compute_byte_size_of_image_stack(512, 512, 1000)
                
            temp_info = [rank, this_node, os.getpid(), tempdir, DiskSpace().humanize_bytes(byte_size)]

            tempdir_record = comm.gather(temp_info, root = 0)
            
            if rank == 0:
                log_str = 'The following temporary directories were made:\n'
                log_tab_str = tabulate(tempdir_record, ['rank_id', 'node_name', 'process_id', 'directory', 'space_required'])
                self.log.ilog(log_str + log_tab_str)
                tempdir_str = '\n'.join(log_tab_str.split('\n')[2:])
            else:
                tempdir_str = None
            tempdir_str = comm.bcast(tempdir_str, root=0)
        else:
            tempdir = None

        return tempdir, tempdir_str, err_msg


    def setup_mpi_and_simultaneous_logging(self, log, logfile_name, temppath, byte_size=None):
        from mpi4py import MPI
        comm = MPI.COMM_WORLD
        rank = comm.Get_rank()
        size = comm.Get_size()
        
        if rank != 0:
            log.__init__()
            log.set_logfile(logfile_name)
        
        log.rank = rank
        comm.barrier()
        
        tempdir, tempdir_record, err_msg = self.make_tempdir_and_log_including_node_name(log, temppath, comm, rank,
        size, byte_size)
        
        if rank == 0:
            self.setup_interrupt(tempdir_record, size, MPI)
        
        comm.barrier()

        comm = self.setup_mpi_abort_upon_exception_on_any_node(comm, tempdir_record, MPI)
        if err_msg is not None:
            raise ValueError(err_msg)
        
        return comm, rank, size, log, tempdir
        
        
class TestingDirectory():
    def __init__(self, progname=None):
        if progname is None:
            self.testdir='testing'
        else:
            if progname.startswith('test'):
                self.testdir = progname
            else:
                self.testdir='testing_{0}'.format(progname)

    def create(self):
        os.mkdir(self.testdir)
        os.chdir(self.testdir)

    def remove(self):
        os.chdir(os.pardir)
        files_to_be_removed = os.listdir(self.testdir)
        [os.remove(os.path.join(self.testdir, each_file)) for each_file in files_to_be_removed]
        shutil.rmtree(self.testdir)
        



# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to examine helix classes to compute their collapsed (1D) and 2D power spectrum and width profile of helices
"""
from EMAN2 import EMData, EMUtil, Util, periodogram
from matplotlib.backends.backend_pdf import PdfPages, FigureCanvasPdf
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import OpenMpi, Temporary
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segmentexam import SegmentExamPar, SegmentExam
import os
from utilities import model_blank


class SegClassExamPar(SegmentExamPar):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'segclassexam'
        self.proginfo = __doc__
        self.code_files = [self.progname, self.progname + '_mpi']

        self.segclassexam_features = Features()
        self.feature_set = self.segclassexam_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    
    def define_parameters_and_their_properties(self):
        self.feature_set = self.segclassexam_features.set_class_avg_stack(self.feature_set)
        self.feature_set = self.segclassexam_features.set_output_plot(self.feature_set, self.progname + '_diag.pdf')

        self.feature_set = self.set_class_var_option(self.feature_set)
        self.feature_set = self.set_class_var_stack(self.feature_set)
        
        self.feature_set = self.segclassexam_features.set_output_power_spectrum(self.feature_set)
        self.feature_set = self.segclassexam_features.set_enhance_power_option(self.feature_set)
        self.feature_set = self.segclassexam_features.set_output_enhanced_power_spectrum(self.feature_set)
        self.feature_set = self.segclassexam_features.set_pixelsize(self.feature_set)
        self.feature_set = self.segclassexam_features.set_helix_width(self.feature_set)
        self.feature_set = self.segclassexam_features.set_power_cutoff(self.feature_set)
        self.feature_set = self.segclassexam_features.set_class_number_range_to_be_analyzed(self.feature_set)
        
        self.feature_set = self.segclassexam_features.set_mpi(self.feature_set)
        self.feature_set = self.segclassexam_features.set_ncpus(self.feature_set)
        self.feature_set = self.segclassexam_features.set_temppath(self.feature_set)


    def define_program_states(self):
        super(SegClassExamPar, self).define_program_states()

        self.feature_set.program_states['prepare_segclassexam']='Prepare classes for analysis.'
        self.feature_set.program_states['perform_class_examination']='Extract classes, profiles and power spectra.'
        self.feature_set.program_states['print_figures']='Save diagnostic plots to file.'


    def set_class_var_option(self, feature_set):
        inp2 = 'Class variance option'
        feature_set.parameters[inp2] = False
        feature_set.hints[inp2] = 'Check if variances of classes are to be analyzed.'
        feature_set.level[inp2]='intermediate'
        
        return feature_set


    def set_class_var_stack(self, feature_set):
        inp2 = 'Class variance stack'
        feature_set.parameters[inp2] = 'variances.hdf'
        feature_set.properties[inp2] = self.file_properties(1, ['spi', 'hdf', 'img', 'hed'], 'getFile')
        feature_set.hints[inp2] = 'Stack: accepted image file formats (%s)' % \
        (', '.join(feature_set.properties[inp2].ext))
        feature_set.level[inp2]='intermediate'
        feature_set.relatives[inp2]='Class variance option'
        
        return feature_set


class SegClassExam(SegmentExam):
    """
    * Class that holds functions for examining segments from micrographs

    * __init__ Function to interpret multi-input parameters

    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.avgstack = p['Class average stack']
            self.infile = self.avgstack
            self.outfile = p['Diagnostic plot']
            self.cls_var_option = p['Class variance option']
            self.varstack = p['Class variance stack']
            
            self.power_img = p['Power spectrum output image']
            self.enhanced_power_option = p['Enhanced power spectrum option']
            self.power_enhanced_img = p['Enhanced power spectrum output image']
            self.ori_pixelsize = p['Pixel size in Angstrom']
            self.helixwidth = p['Estimated helix width in Angstrom']
            self.helixwidthpix = int(round(self.helixwidth / self.ori_pixelsize))
            self.rescutoff = p['Power spectrum resolution cutoff in 1/Angstrom']
            self.classno_range = p['Class number range to be analyzed']

            self.binfactor = int(round(1/(self.rescutoff * self.ori_pixelsize * 2)))

            self.stack = EMData()
            self.stack.read_image(self.avgstack, 0)
            self.segsizepix = self.stack.get_xsize()

            self.avgstack, self.segsizepix, self.helixwidthpix, self.pixelsize = \
            SegmentExam().apply_binfactor(self.binfactor, self.avgstack, self.segsizepix, self.helixwidthpix,
            self.ori_pixelsize)
                            
            if self.cls_var_option:
                self.varstack, segsizepix, helixwidthpix, pixelsize = \
                SegmentExam().apply_binfactor(self.binfactor, self.varstack, self.segsizepix, self.helixwidthpix, 
                self.ori_pixelsize)

            self.mpi_option = p['MPI option']
            self.cpu_count = p['Number of CPUs']
            self.temppath = p['Temporary directory']


    def display_helix_width_and_normal_profile(self, avg, var):
        widthavg = self.project_helix(avg)
        widthvar = self.project_helix(var)
        self.ax2.set_title('Helix width profile', fontsize=8)
        self.ax2 = self.add_width_profile_from_avg_and_var(widthavg, widthvar, self.pixelsize, self.ax2)
        self.log.ilog('Width profile of average segment included in montage')
        
        vertical_avg = self.project_normal_to_helix(avg)
        vertical_var = self.project_normal_to_helix(var)
        self.ax4.set_title('Helix normal profile', fontsize=8)
        self.ax4 = self.add_width_profile_from_avg_and_var(vertical_avg, vertical_var, self.pixelsize, self.ax4, 'normal')
        self.log.ilog('Normal profile of average segment included in montage')

    
    def check_maximum_class_number(self, avgstack, classno_range):
        max_class_number = EMUtil.get_image_count(avgstack) - 1
        if max_class_number < classno_range[1]:
            msg = 'Class average stack does not have specified class number. Therefore class number range is ' + \
            'corrected to available class numbers, i.e. {0}.'.format(max_class_number)
            
            classno_range = ((classno_range[0], max_class_number))
            self.log.wlog(msg)
            
        return classno_range


    def prepare_segclassexam(self):
        self.log.fcttolog()
        self.helixmask = self.make_smooth_rectangular_mask(self.helixwidthpix, 0.7 * self.segsizepix, self.segsizepix, 
        width_falloff=0.15)

        self.classno_range = self.check_maximum_class_number(self.avgstack, self.classno_range)
        classno_start, classno_end = self.classno_range
        self.log.plog(10)

        return classno_start, classno_end


    def rename_plot_title_for_multiple_classes(self, infile, outfile, classno_start, classno_end, each_class_index,
    feature_set):
        class_file = '{0}_class{1:04}{2}'.format(os.path.splitext(infile)[0], each_class_index,
        os.path.splitext(infile)[-1])
        
        plot_file = '{0}_class{1:04}{2}'.format(os.path.splitext(outfile)[0], each_class_index,
        os.path.splitext(outfile)[-1])
        
        feature_set.parameters[list(feature_set.parameters.keys())[0]] = class_file
        feature_set.parameters[list(feature_set.parameters.keys())[1]] = plot_file
                
        return plot_file, feature_set


    def perform_class_examination(self, avgstack, varstack, classno_start, classno_end, power_img, power_enhanced_img):
        self.log.fcttolog()

        figures = []
        avg = EMData()
        var = EMData()
        
        classes_iter = list(range(classno_start, classno_end + 1))
        for each_cls_id, each_class in enumerate(classes_iter):
            if len(classes_iter) > 1 or hasattr(self, 'comm'):
                plot_file, self.feature_set = self.rename_plot_title_for_multiple_classes(self.infile, self.outfile,
                classno_start, classno_end, each_class, self.feature_set)
            else:
                plot_file = self.outfile

            fig = self.setup_fourxtwo()
            avg.read_image(avgstack, each_class)
            avg *= self.helixmask

            if self.cls_var_option:
                var.read_image(varstack, each_class)
            else:
                var = model_blank(avg.get_xsize(), avg.get_ysize(), 1)

            self.display_average_and_variance(avg, var)
            self.display_helix_width_and_normal_profile(avg, var)

            padsize = 4 * avg.get_xsize()
            padded_avg = Util.pad(avg, padsize, padsize, 1, 0, 0, 0, '0')
            avg_periodogram = periodogram(padded_avg)
            avg_periodogram.set_attr('cls', each_class)
            avg_periodogram.write_image(power_img, each_cls_id)

            avg_periodogram_enhanced = self.enhance_power(avg_periodogram, self.pixelsize)
            if self.enhanced_power_option:
                avg_periodogram_enhanced.set_attr('cls', each_class)
                avg_periodogram_enhanced.write_image(power_enhanced_img, each_cls_id)

            self.log.plog(30 * (each_class + 0.5 - 1) / (classno_end + 1) + 10)

            avg_collapsed_power_line = self.collapse_power(avg_periodogram)
            avg_collapsed_line_enhanced = self.collapse_power(avg_periodogram_enhanced)

            self.display_power_spectra_enhanced_and_collapsed(avg_periodogram, avg_periodogram_enhanced,
            avg_collapsed_power_line, avg_collapsed_line_enhanced)

            figures.append((fig, plot_file))
            self.log.plog(30 * (each_class) / (classno_end + 1) + 10)
        
        return figures


    def print_figures(self, figures):
        self.log.fcttolog()
        if os.path.splitext(self.outfile)[-1].endswith('pdf') and not hasattr(self, 'comm'):
            pdf = PdfPages(self.outfile)

        for each_id, (each_fig, each_plot_file) in enumerate(figures):
            if os.path.splitext(self.outfile)[-1].endswith('pdf') and not hasattr(self, 'comm'):
                canvas = FigureCanvasPdf(each_fig) 
                each_fig.savefig(pdf, format='pdf')
            elif not os.path.splitext(self.outfile)[-1].endswith('pdf') and len(figures) > 1 or \
            os.path.splitext(self.outfile)[-1].endswith('pdf') and hasattr(self, 'comm'):
                each_fig.savefig(each_plot_file, dpi=600)
            else:
                each_fig.savefig(self.outfile, dpi=600)
            self.log.ilog('The diagnostic plot {0} was added.'.format(each_plot_file))
            self.log.plog(60 * (each_id) / (len(figures)) + 40)

        if os.path.splitext(self.outfile)[-1].endswith('pdf') and not hasattr(self, 'comm'):
            pdf.close()
            self.log.ilog('The diagnostic plot {0} was saved.'.format(self.outfile))


    def print_figures_and_finish(self, figures):
        self.print_figures(figures)
        os.rmdir(self.tempdir)
        self.log.endlog(self.feature_set)


    def exam_classes(self):
        OpenMpi().setup_and_start_mpi_version_if_demanded(self.mpi_option, self.feature_set, min(self.cpu_count, 
        self.classno_range[-1] - self.classno_range[0] + 1))

        self.tempdir = Temporary().mktmpdir(self.temppath)

        classno_start, classno_end = self.prepare_segclassexam()

        figures = self.perform_class_examination(self.avgstack, self.varstack, classno_start, classno_end,
        self.power_img, self.power_enhanced_img)

        self.print_figures_and_finish(figures)


def main():
    # Option handling
    parset = SegClassExamPar()
    mergeparset = OptHandler(parset)

    ######## Program
    stack = SegClassExam(mergeparset)
    stack.exam_classes()

if __name__ == '__main__':
    main()



# Author: Carsten Sachse 11-Nov-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.micprgs.micctfdetermine_mpi import ScanMpi
from spring.micprgs.scansplit import ScanSplit, ScanSplitPar

class ScanSplitMpi(ScanSplit, ScanMpi):

    def perform_splitscan(self):
        self.startup_scan_mpi_programs()
        
        if self.micrograph_files != []:
            self.perform_splitscan_by_finding_location_with_respect_to_reference_micrograph(self.micrograph_files,
            self.outfiles)
        
        self.end_scan_mpi_programs()

def main():
    parset = ScanSplitPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    micrograph = ScanSplitMpi(reduced_parset)
    micrograph.perform_splitscan()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segment module
"""
from EMAN2 import EMData, EMUtil, Util, EMAN2Ctf, EMNumPy
from filter import filt_ctf
from fundamentals import fshift, rot_shift2D
from glob import glob
from multiprocessing import cpu_count
import os
import shutil
from spring.csinfrastr.csdatabase import SpringDataBase, base, SegmentTable
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.micctfdetermine import MicCtfDetermine
from spring.segment2d.segment import Segment, SegmentPar
from spring.segment2d.segmentexam import SegmentExam
from spring.springgui.spring_launch import DataBaseUpdate
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentexam import HelixProjectionPreparation
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dMain
from utilities import model_gauss_noise, model_blank

from nose.tools import raises
from scipy.interpolate.fitpack2 import InterpolatedUnivariateSpline
from scipy.ndimage.interpolation import map_coordinates

import numpy as np


class MicrographHelixPreparation(HelixProjectionPreparation):
    def prepare_ctf_micrograph(self, micrograph_file, avg_defocus, cs, voltage, pixelsize, ampcont, astigmatism,
    astigmatism_angle, helix=True):
        if helix:
            micrograph = MicrographHelixPreparation().prepare_dummy_helix()
        else:
            micrograph = model_blank(1000, 1000)
        dimension = micrograph.get_xsize()
        offset = (dimension - 1000) / 2
        micrograph = Util.pad(micrograph, 1000, 1000, 1, offset, offset, 0, '0')
        noise = model_gauss_noise(0.1, 1000, 1000)
        micrograph += noise
        
        ctf = EMAN2Ctf()
        
        ctf.from_dict({'defocus':avg_defocus * 1e-4, 'cs':cs, 'voltage':voltage, 'apix':pixelsize, 'ampcont':ampcont,
        'bfactor':0.0, 'dfdiff':astigmatism * 1e-4 , 'dfang':astigmatism_angle})
        
        ctf_micrograph = filt_ctf(micrograph, ctf)
        ctf_micrograph.write_image(micrograph_file)
        
    def prepare_helix_coordinate_file(self, filename='test_mic.box'):
        self.segment_size_in_pixel = 100
        self.helix_coordinate_file = filename

        if os.path.exists(filename):
            os.remove(filename)

        if self.helix_coordinate_file.endswith('box'):
            Segment().write_boxfile(self.x_coordinates_array, self.y_coordinates_array, self.segment_size_in_pixel,
            self.helix_coordinate_file)
            
        elif self.helix_coordinate_file.endswith('star'):
            self.prepare_helix_coordinate_bsoft_file(self.x_coordinates_array, self.y_coordinates_array,
            self.segment_size_in_pixel, self.helix_coordinate_file)


    def add_segmented_particle_entry(self, x_coordinates, y_coordinates, segment_size_in_pixel):
        lines = '_particle.select\n'
        angles = np.rad2deg(np.arctan(x_coordinates / y_coordinates))
        for each_index, each_x_coordinate in enumerate(x_coordinates):
            each_y_coordinate = y_coordinates[each_index]
            each_angle = angles[each_index]
            
            particle_coordinate_line = \
            '{id:5}{hel_id:5}{zero:8}{one:8.4f}{x_coord:8.2f}{y_coord:8.2f}{zero:8.2f}{size:8.3f}{size:8.3f}{zero:8.3f}{one:8.4f}{zero:8.4f}{zero:8.4f}{angle:8.2f}{one:8.4f}{hel_id:5}\n'.\
            format(id=each_index + 1, hel_id=1, zero=0, one=1.0, x_coord=each_x_coordinate, y_coord=each_y_coordinate,
            size=segment_size_in_pixel / 2, angle=each_angle)
            
            lines += particle_coordinate_line
        
        end_lines = '\n\nloop_\n'
        lines += end_lines
        
        return lines
    

    def add_filament_entry(self, lines, x_coordinates, y_coordinates):
        more_lines = '_filament.id\n_filament.node_id\n_filament.x\n_filament.y\n_filament.z\n'
        lines += more_lines
        filament_coordinate_line_start = '{hel_id:7}{point_id:7}{x_coord:8.2f}{y_coord:8.2f}{zero:8.2f}\n'.\
        format(hel_id=1, point_id=1, x_coord=x_coordinates[0], y_coord=y_coordinates[0], zero=0.0)
        
        filament_coordinate_line_end = '{hel_id:7}{point_id:7}{x_coord:8.2f}{y_coord:8.2f}{zero:8.2f}\n'.\
        format(hel_id=1, point_id=2, x_coord=x_coordinates[-1], y_coord=y_coordinates[-1], zero=0.0)
        
        lines += filament_coordinate_line_start
        lines += filament_coordinate_line_end
        lines += '\n\n'
        
        return lines
    


    def prepare_bsoft_lines(self, x_coordinates, y_coordinates, segment_size_in_pixel):
        """
        >>> from spring.tests.segment2d.test_segment import MicrographHelixPreparation
        >>> m = MicrographHelixPreparation()
        >>> m.prepare_bsoft_lines(np.arange(1000.0, 1005.0), np.arange(1000.0, 1005.0), 600) #doctest: +NORMALIZE_WHITESPACE
        '_particle.select\\n    
        1    1       0  1.0000 1000.00 1000.00    0.00 300.000 300.000   0.000  1.0000  0.0000  0.0000   45.00  1.0000    1\\n    
        2    1       0  1.0000 1001.00 1001.00    0.00 300.000 300.000   0.000  1.0000  0.0000  0.0000   45.00  1.0000    1\\n    
        3    1       0  1.0000 1002.00 1002.00    0.00 300.000 300.000   0.000  1.0000  0.0000  0.0000   45.00  1.0000    1\\n    
        4    1       0  1.0000 1003.00 1003.00    0.00 300.000 300.000   0.000  1.0000  0.0000  0.0000   45.00  1.0000    1\\n    
        5    1       0  1.0000 1004.00 1004.00    0.00 300.000 300.000   0.000  1.0000  0.0000  0.0000   45.00  1.0000    
        1\\n\\n\\nloop_\\n_filament.id\\n_filament.node_id\\n_filament.x\\n_filament.y\\n_filament.z\\n      
        1      1 1000.00 1000.00    0.00\\n      1      2 1004.00 1004.00    0.00\\n\\n\\n'
        """
        lines = self.add_segmented_particle_entry(x_coordinates, y_coordinates, segment_size_in_pixel)
        lines = self.add_filament_entry(lines, x_coordinates, y_coordinates)
        
        return lines
    

    def prepare_helix_coordinate_bsoft_file(self, x_coordinates, y_coordinates, segment_size_in_pixel, filename):
        
        lines = self.prepare_bsoft_lines(x_coordinates, y_coordinates, segment_size_in_pixel)
        
        bsoft_file = open(filename, 'w')
        bsoft_file.write(lines)
        bsoft_file.close()
        
        return lines
    
        
    def prepare_dummy_helix(self, angle=45, shift_in_angstrom=30):
        self.width_of_helix_in_angstrom = 300
        self.micrograph_size_in_angstrom = 1000
        self.pixelsize = 5
        self.width_of_helix_in_pixel = self.width_of_helix_in_angstrom / self.pixelsize
        self.micrograph_size_in_pixel = self.micrograph_size_in_angstrom / self.pixelsize
        self.segment_size_in_pixels = self.micrograph_size_in_pixel
        self.make_empty_segment()
        self.compute_intersect_sine_waves()
        self.helix_segment_img = self.add_intersects_to_segment_image()
        
        self.helix_micrograph = self.helix_segment_img
        self.shift_in_pixel = shift_in_angstrom / self.pixelsize

        self.helix_micrograph = fshift(self.helix_micrograph, self.shift_in_pixel)
        if angle != 0:
            self.helix_micrograph = rot_shift2D(self.helix_micrograph, angle)

        return self.helix_micrograph


    def prepare_helix_on_micrograph(self):
        """
        * Function to prepare a dummy helix on micrograph
        """
        self.prepare_dummy_helix()
        
        self.add_noise_to_helix_micrograph()
        
        self.micrograph_test_file = 'test_mic.hdf'
        self.test_frame_count = 5
        self.helix_micrograph.write_image(self.micrograph_test_file)
        self.micrograph_test_file = os.path.abspath('test_mic.hdf')

        self.x_coordinates_array = self.y_coordinates_array = np.array([100, 90, 80, 70, 60])
        self.prepare_helix_coordinate_file()
        
        return self.helix_micrograph
    

    def setup_helix_or_particle_dimensions(self):
        self.feature_set.parameters['Segment coordinates'] = self.helix_coordinate_file
        self.feature_set.parameters['Estimated helix width in Angstrom'] = int(1.2 * self.width_of_helix_in_angstrom)
        self.feature_set.parameters['Segment size in Angstrom'] = int(self.segment_size_in_pixel * self.pixelsize)
        self.feature_set.parameters['Step size of segmentation in Angstrom'] = int(50)
        self.feature_set.parameters['Perturb step option'] = False
        self.feature_set.parameters['Rotation option'] = False
        self.feature_set.parameters['Unbending option'] = False
        self.feature_set.parameters['Remove helix ends option'] = False
        
        
    def setup_of_common_non_mpi_parameters(self):
        self.feature_set.parameters['Micrographs'] = self.micrograph_test_file
        self.feature_set.parameters['Image output stack'] = 'test_output_stack.hdf'
        self.feature_set.parameters['Spring database option'] = True
        self.feature_set.parameters['spring.db file'] = os.path.join(os.pardir, 'spring.db')
            
        self.feature_set.parameters['Pixel size in Angstrom'] = float(self.pixelsize)
        self.setup_helix_or_particle_dimensions()
        self.feature_set.parameters['CTF correct option'] = False
        self.feature_set.parameters['CTFFIND or CTFTILT'] = 'ctftilt'
        self.feature_set.parameters['convolve or phase-flip'] = 'phase-flip'  # 'convolve'
        self.feature_set.parameters['Astigmatism correction'] = True
        
        self.feature_set.parameters['Frame processing option'] = False
        self.feature_set.parameters['First and last frame'] = (0, self.test_frame_count - 1)
        self.feature_set.parameters['Refinement.db to process'] = 'refinement.db'

        self.feature_set.parameters['Invert option'] = False
        self.feature_set.parameters['Normalization option'] = True
        self.feature_set.parameters['Row normalization option'] = True
        self.feature_set.parameters['Binning option'] = True
        self.feature_set.parameters['Binning factor'] = 6
        
        self.setup_of_selection_parameters()
        
        
    def setup_of_selection_parameters(self):
        self.feature_set.parameters['Micrographs select option'] = False
        self.feature_set.parameters['Include or exclude micrographs'] = 'include'
        self.feature_set.parameters['Micrographs list'] = '1'
        
        self.feature_set.parameters['Helices select option'] = False
        self.feature_set.parameters['Include or exclude helices'] = 'include'
        self.feature_set.parameters['Helices list'] = '1'
        
        self.feature_set.parameters['Straightness select option'] = False
        self.feature_set.parameters['Include or exclude straight helices'] = 'include'
        self.feature_set.parameters['Persistence length range'] = (0, 80)
        
        self.feature_set.parameters['Defocus select option'] = False
        self.feature_set.parameters['Include or exclude defocus range'] = 'include'
        self.feature_set.parameters['Defocus range'] = (0, 23000)
        
        self.feature_set.parameters['Astigmatism select option'] = False
        self.feature_set.parameters['Include or exclude astigmatic segments'] = 'include'
        self.feature_set.parameters['Astigmatism range'] = (0, 10000)
        

class TestSegmentComparison(MicrographHelixPreparation, Segment):
#     def compare_centered_segments(self):
#         linear_fit_dummy = np.polyfit(self.x_coordinates_array, self.y_coordinates_array, 1)
# 
#         self.get_centered_coordinates()
#         linear_fit_centered = np.polyfit(self.centered_x_coordinates_array, self.centered_y_coordinates_array, 1)
#         distance_between_old_and_new_coordinates = abs(linear_fit_centered[1] - linear_fit_dummy[1])*np.sin(np.pi/4)
# #        sys.stderr.write('{0}{1}{2}'.format(linear_fit_dummy, linear_fit_centered, distance_between_old_and_new_coordinates))
#         sys.stderr.write('{0}<{1}<{2}'.format(self.shift_in_pixel - 0.8, distance_between_old_and_new_coordinates,
#         self.shift_in_pixel + 0.8))
#         assert self.shift_in_pixel - 0.8 < distance_between_old_and_new_coordinates < self.shift_in_pixel + 0.8
# 
#     def get_centered_coordinates(self):
#         self.centered_x_coordinates_array, self.centered_y_coordinates_array = \
#                         list(zip(*self.helices[0].coordinates))

    def check_number_of_segments_equals_one(self):
        number_of_segments = EMUtil.get_image_count('test_output_stack.hdf')
        assert number_of_segments == 1

#     def compute_difference_between_centered_and_uncentered(self):
#         self.get_centered_coordinates()
#         self.avg_difference_x = abs(np.mean(self.centered_x_coordinates_array) - np.mean(self.x_coordinates_array))
#         self.avg_difference_y = abs(np.mean(self.centered_y_coordinates_array) - np.mean(self.y_coordinates_array))


class TestSegment(TestSegmentComparison):
    def setup_no_helix_generation(self):
        self.feature_set = SegmentPar()
        self.setup_of_common_non_mpi_parameters()
        self.cpu_count = min(8, cpu_count())
        self.feature_set.parameters['MPI option'] = False
        self.feature_set.parameters['Number of CPUs'] = self.cpu_count
        super(TestSegment, self).__init__(self.feature_set)
        

    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.prepare_helix_on_micrograph()
        self.setup_no_helix_generation()
        
#        self.tscd = TestMicCtfDetermine()
#        self.tscd.setup()
#        self.tscd.do_test_case_scd1()
        
#        os.rename('spring.db', os.path.join(os.path.pardir, 'spring.db'))
        
        self.session = SpringDataBase().setup_sqlite_db(base, os.path.join(os.pardir, 'spring.db'))
        self.session.commit()
        self.session.close()

        
    def prepare_spring_database_ctf_entries(self):
        self.pixelsize = 5.0
        self.avg_defocus = 40000.0
        self.voltage = 200
        self.cs = 2.0
        self.ampcont = 0.1
        self.astigmatism = 5000.0
        self.astigmatism_angle = 33.0
        
        ctf_params = MicCtfDetermine().make_ctf_parameter_named_tuple()
        ctf_parameters = ctf_params._make([self.voltage, self.cs, self.ampcont, self.pixelsize])
        
        ctffind_params = MicCtfDetermine().make_ctffind_parameters_named_tuple()
        ctffind_parameters = ctffind_params._make([35000, 45000, 33.0, 10000, 4.5])
        
        ctftilt_params = MicCtfDetermine().make_ctftilt_parameters_named_tuple()
        ctftilt_parameters = ctftilt_params._make([36000, 44000, 30.0, 0, 0, 10000, 300, 300])
        
        session = SpringDataBase().setup_sqlite_db(base, os.path.join(os.pardir, 'spring.db'))
        
        session = MicCtfDetermine().enter_ctffind_values_in_database(session, os.path.abspath(self.micrograph_test_file),
        self.pixelsize, ctf_parameters, ctffind_parameters)
        
        session = MicCtfDetermine().enter_ctftilt_values_in_database(session, os.path.abspath(self.micrograph_test_file),
        self.pixelsize, ctftilt_parameters)
        
        session.commit()
        session.close()
        
        
    def remove_coordfile_micrographdir_and_linkfiles(self, helix_coordinate_file, micrograph_test_file):
        os.remove(helix_coordinate_file)
        os.remove('test_output_stack.hdf')
        micrograph_test_file_base = os.path.basename(micrograph_test_file)
        micrograph_directory = os.path.join( os.path.dirname(micrograph_test_file), micrograph_test_file_base.split(os.extsep)[0])
        files_in_dir = glob('{0}{1}*'.format(micrograph_directory, os.sep))
        for each_file in files_in_dir:
            os.remove(each_file)
        os.rmdir(micrograph_directory)
        

    def teardown(self):
        database_file = os.path.join(os.path.pardir, 'spring.db')
        os.remove(database_file)
        try:
            os.remove(os.path.basename(database_file))
            os.remove(self.micrograph_test_file)
        except:
            comment = 'This is only relevant for running test_segmentctfapply'
         
        os.remove('test_output_stack-6xbin.hdf')
         
        self.remove_coordfile_micrographdir_and_linkfiles(self.helix_coordinate_file, self.micrograph_test_file)
 
        self.testingdir.remove()
        

class TestSegmentMain(TestSegment):
    def do_test_case_seg1(self):
        """
        Segment test box files that contain a set of overlapping boxes
        """
        self.segment()


class TestSegmentMore(TestSegment):
    def do_test_case_seg0(self):
        """
        Segment test box files that contain a set of overlapping boxes including segment database update
        """
        self.segment()
        DataBaseUpdate().update_spring_db('spring.db', 'spring_up.db')
        os.remove('spring_up.db')
 
 
    def do_test_case_seg2(self):
        """
        Segment test box files that contain only start and end point of helix
        """
        self.x_coordinates_array = self.y_coordinates_array = np.array([100, 60])
        self.prepare_helix_coordinate_file()
        self.segment()
          
 
    def do_test_case_seg3(self):
        """
        Segment test box files with a segmentation stepsize of 0
        """
        self.feature_set.parameters['Step size of segmentation in Angstrom'] = 0
        super(TestSegment, self).__init__(self.feature_set)
        self.x_coordinates_array = self.y_coordinates_array = np.array([100, 60])
        self.prepare_helix_coordinate_file()
        self.segment()
        self.check_number_of_segments_equals_one()
 
 
    def do_test_case_seg4(self):
        """
        Segment test box files that are aligned with micrograph columnns
        """
        self.helix_micrograph = rot_shift2D(self.helix_micrograph, -45)
        self.helix_micrograph.write_image(self.micrograph_test_file)
        self.x_coordinates_array = np.array([100, 100, 100, 100, 100])
        self.y_coordinates_array = np.array([100, 90, 80, 70, 60])
        self.prepare_helix_coordinate_file()
        self.segment()
          
  
    def do_test_case_seg5(self):
        """
        Segment test box files that are aligned with micrograph rows
        """
        self.helix_micrograph = rot_shift2D(self.helix_micrograph, 45)
        self.helix_micrograph.write_image(self.micrograph_test_file)
        self.x_coordinates_array = np.array([100, 90, 80, 70, 60])
        self.y_coordinates_array = np.array([100, 100, 100, 100, 100])
        self.prepare_helix_coordinate_file()
        self.segment()
          
  
    def do_test_case_seg6(self):
        """
        Segment test bsoft star files that contain a set of overlapping boxes
        """
        self.x_coordinates_array = self.y_coordinates_array = np.array([100, 90, 80, 70, 60])
        self.star_filename = 'test_mic.star'
        self.prepare_helix_coordinate_file(self.star_filename)
        self.feature_set.parameters['Segment coordinates'] = self.star_filename
        super(TestSegment, self).__init__(self.feature_set)
          
        current_directory = os.path.abspath(os.curdir)
        self.directory = 'test_segmentdir'
        os.mkdir(self.directory)
        os.chdir(self.directory)
        self.segment()
        os.chdir(current_directory)
        EndToEndTest().cleanup_working_directory(self.directory)
          
        os.remove('test_mic.box')
          
    def do_test_case_seg7(self):
        """
        Segment test bsoft star filament coordinates that contain a set of overlapping boxes
        """
        self.x_coordinates_array = self.y_coordinates_array = np.array([100, 90, 80, 70, 60])
        self.star_filename = self.helix_coordinate_file = 'test_mic.star'
        filament_lines = self.add_filament_entry('', self.x_coordinates_array, self.y_coordinates_array)
        coord_file = open(self.star_filename, 'w')
        coord_file.write(filament_lines)
        coord_file.close()
          
        self.feature_set.parameters['Segment coordinates'] = self.star_filename
        super(TestSegment, self).__init__(self.feature_set)
          
        current_directory = os.path.abspath(os.curdir)
        self.directory = 'test_segmentdir'
        os.mkdir(self.directory)
        os.chdir(self.directory)
        self.segment()
        os.chdir(current_directory)
        EndToEndTest().cleanup_working_directory(self.directory)
          
        os.remove('test_mic.box')
  
  
    def do_test_case_seg8(self):
        """
        Segment test Remove helix ends true
        """
        self.feature_set.parameters['Remove helix ends option'] = True
        self.feature_set.parameters['Spring database option'] = False
        self.feature_set.parameters['spring.db file'] = ''
        super(TestSegment, self).__init__(self.feature_set)
          
        self.x_coordinates_array = self.y_coordinates_array = np.arange(150, 0, -10)
        self.prepare_helix_coordinate_file()
        self.segment()
          
          
    def do_test_case_seg9(self):
        """
        Segment test Perturb step option
        """
        self.feature_set.parameters['Perturb step option'] = True
        super(TestSegment, self).__init__(self.feature_set)
          
        self.segment()
          
          
    @raises(ValueError)
    def do_test_case_impossible_seg10(self):
        """
        Segment test to raise error in case no CTF info available
        """
        self.feature_set.parameters['CTF correct option'] = True
        super(TestSegment, self).__init__(self.feature_set)
          
        for each_file in ['test_output_stack.hdf', 'test_output_stack-6xbin.hdf']:
            dummy = open(each_file, 'w')
            dummy.close()
              
        self.segment()
 
         
    def do_test_case_seg99(self):
        """
        Segment test reading coordinates from spring.db including helix selection and rotation option
        """
        self.segment()
        os.rename('spring.db', 'spring_inp.db')
        self.feature_set.parameters['Segment coordinates'] = 'spring_inp.db'
        self.feature_set.parameters['Helices select option'] = True
        self.feature_set.parameters['Rotation option'] = True
        super(TestSegment, self).__init__(self.feature_set)
        self.segment()
        os.remove('spring_inp.db')


class TestSegmentFrames(TestSegment):

    def setup_of_databases_for_frame_processing_from_segmentrefine3d(self):
        self.segment()
        os.remove('spring.db')
        tsr3d = TestSegmentRefine3dMain()
        tsr3d.setup()
        tsr3d.do_test_case_sr1()

        helix_list = [int(each_val) for each_val in self.feature_set.parameters['Helices list'].split(',')]
        session = SpringDataBase().setup_sqlite_db(base, 'spring.db')
        sel_segments = session.query(SegmentTable).filter(SegmentTable.helix_id.in_(helix_list)).all()
        stack_ids = [each_segment.stack_id for each_segment in sel_segments]

        img = EMData()
        self.test_frame_stack = os.path.abspath(os.path.join(os.pardir, tsr3d.input_stack)) 
        for each_frame in list(range(self.test_frame_count)):
            for each_stack_id in stack_ids:
                img.read_image(tsr3d.input_stack, each_stack_id)
                img.append_image(self.test_frame_stack)

        shutil.copy('spring.db', os.path.join(os.pardir, os.pardir, 'spring.db'))
        os.rename('refinement006.db', os.path.join(os.pardir, 'refinement006.db'))
        tsr3d.teardown()
        
        return tsr3d


#     def do_test_case_seg100(self):
#         """
#         Segment test frame processing of spring.db/refinement.db entries
#         """
#         self.setup_of_databases_for_frame_processing_from_segmentrefine3d()
#       
#         mics = ['test_mic_{0:03}.hdf'.format(each_time) for each_time in list(range(5))]
#         [shutil.copy('test_mic.hdf', each_mic) for each_mic in mics]
#             
#         self.feature_set.parameters['Micrographs'] = 'test_mic_???.hdf'
#         self.feature_set.parameters['Segment coordinates'] = os.path.join(os.pardir, 'spring.db')
#         self.feature_set.parameters['Frame processing option']=True
#         self.feature_set.parameters['Refinement.db to process']='refinement006.db'
#         super(TestSegment, self).__init__(self.feature_set)
#         self.segment()
#                     
#         os.rename(self.spring_db_frames, os.path.join(os.pardir, 'spring.db'))
#         [os.remove(each_file) for each_file in [self.ref_db_frames] + mics ]
#       
#         
    def create_mic_mrc_stack(self, mic, file_name=None):
        img = EMData()
        img.read_image(mic)
        img_np = EMNumPy.em2numpy(img.copy())
        
        vol_np = np.zeros((self.test_frame_count, img.get_xsize(), img.get_ysize()))
        for each_plane in range(self.test_frame_count):
            vol_np[each_plane] = img_np
            
        stack = EMNumPy.numpy2em(np.copy(vol_np))
        if file_name is None:
            mrc_stack = os.path.splitext(mic)[0] + os.extsep + 'mrcs'
        else:
            mrc_stack = file_name
        stack.write_image(mrc_stack[:-1])
        
        os.rename(mrc_stack[:-1], mrc_stack)

        return mrc_stack


    def do_test_case_seg101(self):
        """
        Segment test frame processing with mrcs (mrc-stack file) of spring.db/refinement.db entries
        """
        tsr3d = self.setup_of_databases_for_frame_processing_from_segmentrefine3d()
 
        fake_mrc_stack = self.create_mic_mrc_stack('test_mic.hdf')
        os.mkdir(tsr3d.testingdir.testdir)
        os.rename(fake_mrc_stack, os.path.join(tsr3d.testingdir.testdir, fake_mrc_stack))
 
        self.feature_set.parameters['Micrographs'] = os.path.join(tsr3d.testingdir.testdir, fake_mrc_stack)
        self.feature_set.parameters['Segment coordinates'] = os.path.join(os.pardir, 'spring.db')
        self.feature_set.parameters['Frame processing option'] = True
        self.feature_set.parameters['Refinement.db to process'] = 'refinement006.db'
  
        self.feature_set.parameters['Helices select option'] = True
  
        super(TestSegment, self).__init__(self.feature_set)
        self.segment()
          
        shutil.copy(self.spring_db_frames, os.path.join(os.pardir, 'spring.db'))
        os.remove(os.path.join(tsr3d.testingdir.testdir, fake_mrc_stack))
        shutil.rmtree(tsr3d.testingdir.testdir)
 

class TestSegmentMoreStraight(TestSegment):
    def do_test_case_seg8_with_straightening_option(self):
        self.feature_set.parameters['Unbending option'] = True
        super(TestSegment, self).__init__(self.feature_set)
        
        helix = MicrographHelixPreparation().prepare_dummy_helix(0, 0)
        self.helix_micrograph = TestSegmentStraight().prepare_bent_helix(helix)
        self.helix_micrograph = rot_shift2D(self.helix_micrograph, -45)
        self.helix_micrograph.write_image(self.micrograph_test_file)
        
        self.x_coordinates_array = np.array([32, 36, 42, 49, 58, 66, 76, 88, 100]) + 50
        self.y_coordinates_array = np.array([90, 77, 66, 56, 47, 38, 32, 25, 20]) + 50
        self.prepare_helix_coordinate_file()
        self.segment()
        

class TestSegmentCtf(TestSegment):
    def do_test_case_seg10(self):
        """
        Segment test CTF correction with box files that contain a set of overlapping boxes
        """
        self.feature_set.parameters['CTF correct option'] = True
        super(TestSegment, self).__init__(self.feature_set)
        
        self.prepare_spring_database_ctf_entries()

        self.segment()
        

class TestSegmentEndToEnd(TestSegment):
    def do_end_to_end_test_seg_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_seg_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)


class TestSegmentMpi(TestSegment):
    def do_end_to_end_test_seg_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs'] = min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)


class TestSegmentStraightPreparation(object):

    def prepare_bent_helix(self, helix_img):
        img_size = helix_img.get_xsize()
        self.polynom_curve = [1 / 200.0, -1.0, 150.0]
        cols, rows, inplane_angles = self.compute_bending_path_row_wise(float(img_size), self.polynom_curve, 0)
        
        bent_helix = self.straighten_segment(helix_img, img_size, cols, rows, inplane_angles, mode='bend')
        
        return bent_helix
        
        
class TestSegmentStraight(TestSegmentStraightPreparation, Segment):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.bent_helix_name = 'test_helix_bent.hdf'
        self.straightened_helix_name = 'test_helix_straightened.hdf'
        helix = MicrographHelixPreparation().prepare_dummy_helix(0, 0)
        helix.write_image('test_helix.hdf')
        
        self.bent_helix = self.prepare_bent_helix(helix)
        self.bent_helix.write_image(self.bent_helix_name)
        
    def do_test_case_straightening_with_verticalized_helix(self):
        img_size = self.bent_helix.get_xsize()
        cols, rows, inplane_angles = self.compute_bending_path_row_wise(float(img_size), self.polynom_curve, 0)
    
        self.straightened_helix = self.straighten_segment(self.bent_helix, img_size, cols, rows, inplane_angles)
        self.straightened_helix.write_image(self.straightened_helix_name)
         
    def do_test_case_straightening_with_45deg_helix(self):
        angle = 45
        x_offset = 45.4
        y_offset = -15.1
        self.bent_helix = rot_shift2D(self.bent_helix, -angle, x_offset, y_offset)
        self.bent_helix.write_image(self.bent_helix_name)
        
        img_size = self.bent_helix.get_xsize()
        cols, rows, inplane_angles = self.compute_bending_path_row_wise(float(img_size), self.polynom_curve, angle)
        cols += x_offset
        rows += y_offset
        
        self.straightened_helix = self.straighten_segment(self.bent_helix, img_size, cols, rows, inplane_angles,
        'straighten')
        
        self.straightened_helix.write_image(self.straightened_helix_name)

        
    def teardown(self):
        os.remove('test_helix.hdf')

        os.remove(self.straightened_helix_name)
        os.remove(self.bent_helix_name)
    
        self.testingdir.remove()


class TestSegmentWarping(object):
    def setup(self):
        img_size = 40.0
        self.linex = np.ones(img_size) * img_size / 2.0
        self.liney = np.arange(0.0, img_size)
        
        parabola_xx = np.linspace(-(img_size / 2), (img_size / 2), img_size) 
        parabola_yy = 0.02 * (parabola_xx) ** 2 + img_size / 2.0
        
        parabola_xx += img_size / 2.0
        
        self.parabola_x, self.parabola_y = Segment().rotate_coordinates_by_angle(parabola_xx, parabola_yy, 270.0,
        img_size / 2.0, img_size / 2.0)

    
    def start_from_center_and_determine_arc_length(self, linex, liney):
        
        first_half_x = linex[:len(linex) / 2] 
        first_half_y = liney[:len(liney) / 2] 
        
        distances = Segment().compute_cumulative_distances_from_start_of_helix(first_half_x, first_half_y)
        
        return distances
    
    
    def interpolate_xy_finely(self, xcoord, ycoord, angle):
        rotx, roty = Segment().rotate_coordinates_by_angle(xcoord, ycoord, angle, xcoord[0], ycoord[0])
        
        rotx_int = np.linspace(rotx[0], rotx[-1], 100 * len(rotx))
        spline = InterpolatedUnivariateSpline(rotx, roty, k=1)
        roty_int = spline(rotx_int)
        
        xcoord_int, ycoord_int = Segment().rotate_coordinates_by_angle(rotx_int, roty_int, -angle,
        xcoord[0], ycoord[0])
        
        return xcoord_int, ycoord_int
    
        
    def do_test_arc_length(self):
        
        rounded_target_x = np.round(self.parabola_x)
        rounded_target_y = np.round(self.parabola_y)
        
        fine_x, fine_y = self.interpolate_xy_finely(self.linex, self.liney, 270.0)
        
        line_distances = self.start_from_center_and_determine_arc_length(fine_x, fine_y)
        parabola_distances = self.start_from_center_and_determine_arc_length(rounded_target_x, rounded_target_y)
        
        source_coord_x = []
        source_coord_y = []
        for each_target_dist in parabola_distances:
            min_ind = np.argmin(np.abs(line_distances - each_target_dist))
            source_coord_x.append(fine_x[min_ind])
            source_coord_y.append(fine_y[min_ind])
        
        
class TestSegmentMapping(object):
    def setup(self):
        self.img_size = 40.0
        self.linex = np.ones(self.img_size) * self.img_size / 2.0
        self.liney = np.arange(0.0, self.img_size)
        
        self.parabola_y = np.linspace(-(self.img_size / 2), (self.img_size / 2), self.img_size) 
        self.parabola_x = 0.02 * (self.parabola_y) ** 2 + self.img_size / 2.0
        
        self.parabola_y += self.img_size / 2.0
        
        
    def insert_x_y_into_image(self, img_size, x_arr, y_arr, map_axis):
        img = np.zeros((img_size, img_size))
        
        for (each_x, each_y) in zip(x_arr, y_arr):
            if int(each_x) < img_size and int(each_y) < img_size:
                img[int(each_y)][int(each_x)] = 1.0
        
        return img
    
            
    def generate_complete_xmap(self):
        complete_y_map = np.vstack([each_col * np.ones(self.img_size) for each_col in np.arange(self.img_size)])
        complete_y_map = np.swapaxes(complete_y_map, 0, 1)
        
        return complete_y_map
    

    def generate_complete_map(self, map_y_img):
        complete_y_map = np.zeros((self.img_size, self.img_size))
        for each_col_id, each_col in enumerate(np.arange(-self.img_size / 2, self.img_size / 2)):
            complete_y_map += np.roll(map_y_img * each_col_id, int(each_col), axis=1)
        
        return complete_y_map


    def do_test_mapping(self):
        map_y_img = self.insert_x_y_into_image(self.img_size, self.parabola_x, self.parabola_y, self.linex)
        
        source_y_map = self.generate_complete_map(map_y_img)
        source_x_map = self.generate_complete_xmap()
            
        helix = SegmentExam().make_smooth_rectangular_mask(3, 40, 40)
        
        helix_arr = np.copy(EMNumPy.em2numpy(helix))
        
        
        zi = map_coordinates(helix_arr, [source_x_map, source_y_map], order=3)
        output = zi.reshape(helix_arr.shape)
    
def main():
    tseg = TestSegmentMain()
    tseg.setup()
    tseg.do_test_case_seg1()
        
if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Author: Carsten Sachse 21-Sep-2010
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from collections import OrderedDict
from getpass import getuser
from spring.csinfrastr.csproductivity import Errors
import logging
import os
import re
import readline
import socket
import sys
import time
import urllib2


class GetMetaData:
    """
    * Class to get metadata from PKG-INFO egg
    """
    def __init__(self, package):
        self.package = package
        self.read_pkginfo()
        self.add_fields()


    def read_pkginfo(self):
        import pkg_resources
        dist = pkg_resources.get_distribution(self.package)
        pkginfo = dist.get_metadata('PKG-INFO')
        self.metadata = OrderedDict()
        for line in pkginfo.split('\n'):
            if ':' in line:
                if 'Classifier' in line:
                    lsplit = line.split(':')
                    k, v = ':'.join(lsplit[:2]), ''.join(lsplit[2:])
                else:
                    lsplit= line.split(':')
                    k, v = lsplit[-2], lsplit[-1]
                last_key = k.strip()
                last_value = v.strip()
                self.metadata[last_key]=last_value


    def add_fields(self):
        # additional entries
        self.metadata['Package'] = self.package
        self.metadata['Maintainer'] = self.metadata['Author']
        self.metadata['Copyright'] = '2010 - %s, EMBL' %time.localtime()[0]


    def release_meta(self):
        return self.metadata


class ParameterFilter(Errors):
    """
    * Class that filters whether specified values are within specified range, and 
    checks whether files exist
    """
    def __init__(self, parset=None):
        if parset is not None:
            self.feature_set = parset
            self.parameters = parset.parameters
            self.default_params = parset.default_params
            self.level = parset.level
            self.properties = parset.properties
            self.relatives = parset.relatives

            self.check_input_parameters_values_and_files()
            

    def check_input_parameters_values_and_files(self):
        from spring.csinfrastr.csfeatures import Features
        self.completion_required = False
        for self.name in self.parameters:
            if type(self.parameters[self.name]) in [int, float]:
                self.check_valerr(self.parameters[self.name])
            elif type(self.parameters[self.name]) is tuple:
                self.check_valerr(self.parameters[self.name][0])
                self.check_valerr(self.parameters[self.name][1])
            elif type(self.parameters[self.name]) is str:
                if self.properties[self.name].__str__().startswith('file'):
                    filelist = Features().convert_list_of_files_from_entry_string(self.parameters[self.name], check_location=False)
                    self.check_fileexistence_and_fileformat(filelist)
            elif self.parameters[self.name] is None and self.level[self.name] in ['beginner']:
                error_message = 'Value for \'{0}\' was not entered or received correctly. '.format(self.name) + \
                'This is an essential parameter. Please enter a value for \'{0}\' and start again.'.format(self.name)
                raise ValueError(error_message)
            elif self.parameters[self.name] is None and not self.level[self.name] in ['beginner']:
                self.completion_required = True
                self.parameters[self.name] = self.default_params[self.name]
                

    def check_fileexistence_and_fileformat(self, filelist):
        for file in filelist:
            if self.properties[self.name].ftype in ['getFile', 'getFiles', 'saveFile']:
                self.check_fileformat(file)
                if self.properties[self.name].ftype not in ['saveFile']:
                    self.check_filerr(file)
            elif self.properties[self.name].ftype in ['getDir']:
                self.check_filerr(file)
                

class MultiCaster(object):
    def __init__(self, filelist):
        self.filelist = filelist

    def write(self, str):
        for f in self.filelist:
            f.write(str)
            

    def writelines(self, str):
        for f in self.filelist:
            f.writelines(str)
            

    def flush(self):
        for f in self.filelist:
            f.flush()


class NameGenerator(object):
    def make_directory(self, program):
        directory_name = '%s_' %(program.split('.')[0]) + time.strftime('%d_%b_%Y_%H_%M_%S') + \
        '_' + '%d' %os.getpid()
        
        return directory_name
    

    def make_logfile(self, program):
        directory_name = self.make_directory(program)
        logfile = directory_name + os.extsep + 'log'
        
        return logfile
        

class Logger(ParameterFilter):
    """
    * Logger class to intitialize logfile and logging information
    """
    def __init__(self):
#        import logging
        self.log = logging.getLogger(getuser())
        
        self.start_time = time.time()
    

    def log_individual_parameters(self, parameters, to_be_printed=False):
        for name in parameters:
            parinfo = '%-40s = %s' % (name, parameters[name])
            self.log.info('{0}'.format(parinfo))
            if to_be_printed:
                print parinfo


    def startlog(self, parset, debug=True):
        """
        * Function to initiate logfile with time and name of program, adds input to header
        """
        self.debug = debug

        self.feature_set = parset
        self.parameters = parset.parameters
        self.properties = parset.properties
        self.relatives = parset.relatives
        self.progname = parset.progname
        self.version = parset.eggmeta['Version']
        self.package = parset.eggmeta['Name'].title()
        self.devstate = parset.eggmeta['Classifier: Development Status']

        options = self.feature_set.parser.parse_args()

        if options.logfilename is None:
            logfile = 'report.log'#NameGenerator().make_logfile(self.progname)
        else:
            logfile = options.logfilename
            
        lgfile=open(logfile,'a')
        mc = MultiCaster([sys.stderr, lgfile])
        sys.stderr = mc

        if self.debug is True or self.devstate.strip()[0] in ['1', '2', '3', '4']:
            logging.basicConfig(level=logging.DEBUG, filename=logfile, filemode='a')
        else:
            logging.basicConfig(level=logging.INFO, filename=logfile, filemode='a')

        self.log.info('\n'+ '#'*100 + '\n' + ' '*5 + 'Beginning of the program ' + self.progname + \
                      ' (' + self.package + ' v' + self.version +')' + ': ' + \
                      time.strftime('%a, %d %b %Y %H:%M:%S', time.localtime())+ '\n' + '#'*100)
        self.log.info('\n' + '#'*44 + ' INPUT ' +'#'*44)

        self.fcttolog()
        receive_statement = 'The following input parameters have been received:'
        print(receive_statement)
        self.log.info(receive_statement)
        self.log_individual_parameters(self.parameters, to_be_printed=True)
        super(Logger, self).__init__(self.feature_set)
        
        if self.completion_required:
            self.log.info('\n' + '-'*100 + '\nThe final set of parameters has been completed with the provided ' + \
                          'default parameters:')
            self.log_individual_parameters(self.parameters)
        self.log.info('\n' + '#'*44 + ' START ' +'#'*44)
        self.plog(1)

        return logfile


    def set_logfile(self, logfile):
        self.log = logging.basicConfig(level=logging.INFO, filename=logfile, filemode='a')


    def fcttolog(self):
        """
        * Function to switch logger idendity to the function calling
        """
        import inspect
        caller = inspect.getouterframes(inspect.currentframe())[1][3]
        if hasattr(self, 'rank'):
            caller = 'CPU{0}:{1}'.format(self.rank, caller)

        self.log = logging.getLogger(caller)
        self.log.info('\n')


    def ilog(self, msg):
        """
        * Function to add info message for log
        """
        self.log.info(msg)


    def in_progress_log(self):
        """
        * Function to add 'in progress' message and time
        """
        self.tlog('\t\tin progress ...')
        

    def plog(self, percent):
        """
        * Function to add 'progress state' message for progress bar
        """
        self.log.info('progress state: {0} %  [{1:41}]'.format(int(percent), '=' * int(percent / 2.5) + '>'))
        

    def tlog(self, msg):
        """
        * Function to add info message to log including an additional time statement
        """
        self.log.info('{0}\n\tlogged on {1}'.format(msg, time.strftime('%a, %d %b %Y %H:%M:%S', time.localtime())))
        

    def dlog(self, msg):
        """
        * Function to add debug message for log
        """
        self.log.debug(msg)


    def wlog(self, msg):
        """
        * Function to add error message for log
        """
        self.log.warning(msg)


    def errlog(self, msg):
        """
        * Function to add error message for log
        """
        self.log.error(msg)


    def check_for_latest_version(self, feature_set):
        version_check = urllib2.urlopen('http://www.google-analytics.com/collect' + \
        '?v=1&tid=UA-46010145-1&cid=123&t=event&ec=Version%20Check&ea=Endlog&el={0}'.format(getuser()), timeout=1).close
        
        version_file = urllib2.urlopen("http://www.sachse.embl.de/emspring/_static/version.txt", timeout=1)
        latest_version = [version_file.read()]
        
        running_version = feature_set.version
        if running_version in latest_version and len(latest_version[0]) < 20:
            pass
        else:
            update_msg = 'Warning: Please update your {0}{1} package '.format(feature_set.package, running_version) + \
                          'as there is a new version {0} for download available '.format(latest_version[0]) + \
                          'from www.sachse.embl.de/emspring/install.html'
            print update_msg
            self.log.warning(update_msg)


    def endlog(self, feature_set):
        """
        * Function to finish log with time and name of program
        """
        if sys.argv[0].endswith(feature_set.progname):
            try:
                self.check_for_latest_version(feature_set)
            except urllib2.URLError or socket.timeout:
                pass
        elif sys.argv[0].endswith('nosetests'):
            pass
        
        total_time = time.time() - self.start_time
        self.fcttolog()
        self.plog(100)
        self.log.info('\n' + '#'*100) 
        cpu_time = time.gmtime(total_time)
        self.log.info('\n' + ' '*20 + 'Elapsed CPU time: ' + \
        '{0} d {1:02}:{2:02}:{3:02}'.format(cpu_time.tm_mday - 1, cpu_time.tm_hour, cpu_time.tm_min, cpu_time.tm_sec))
        
        self.log.info('\n' + '#'*100 + '\n' + ' '*20 + 'End of program ' + '%s: ' %feature_set.progname + \
                      time.strftime('%a, %d %b %Y %H:%M:%S', time.localtime())+ '\n' + '#'*100)


class FileCompleter(object):
    def _listdir(self, root):
        "List directory 'root' appending the path separator to subdirs."
        res = []
        for name in os.listdir(root):
            path = os.path.join(root, name)
            if os.path.isdir(path):
                name += os.sep
            res.append(name)
        return res


    def _complete_path(self, path=None):
        "Perform completion of filesystem path."
        if not path:
            return self._listdir('.')
        dirname, rest = os.path.split(path)
        tmp = dirname if dirname else '.'
        res = [os.path.join(dirname, p)
                for p in self._listdir(tmp) if p.startswith(rest)]
        # more than one match, or single match which does not exist (typo)
        if len(res) > 1 or not os.path.exists(path):
            return res
        # resolved to a single directory, so return list of files below it
        if os.path.isdir(path):
            return [os.path.join(path, p) for p in self._listdir(path)]
        # exact file match terminates this completion
        return [path + ' ']


    def complete_file(self, args):
        "Completions for the 'extra' command."
        if not args:
            return self._complete_path(path=None)
        # treat the last arg as a path and complete it
        return self._complete_path(args[-1])


    def complete(self, text, state):
        "Generic readline completion entry point."
        buffer = readline.get_line_buffer()
        line = readline.get_line_buffer().split()
        args = line

        return (self.complete_file(args) + [None])[state]


class InputCompleter(object):
    """
    http://stackoverflow.com/questions/5637124/tab-completion-in-pythons-raw-input
    """
    def __init__(self, words):
        self.words = words
        self.re_space = re.compile('.*\s+$', re.M)

    def complete(self, text, state):
        "Generic readline completion entry point."
        buffer = readline.get_line_buffer()
        line = readline.get_line_buffer().split()
        # show all words
        if not line:
            return [c + ' ' for c in self.words][state]
        # account for last argument ending in a space
        if self.re_space.match(buffer):
            line.append('')
        # resolve command to the implementation function
        cmd = line[0].strip()
        if cmd in self.words:
            impl = getattr(self, 'complete_%s' % cmd)
            args = line[1:]
            if args:
                return (impl(args) + [None])[state]
            return [cmd + ' '][state]
        results = [c + ' ' for c in self.words if c.startswith(cmd)] + [None]
        return results[state]

    